capture log close
clear all
macro drop _all
version 14.2
cd "C:/EconometricCapitalisationForAER"
set more off
log using anBenchReg2-long, replace text
*set scheme vg_s1c
set maxiter 200
*ssc inst _gwtmean // used to do weighted mean

/*--------------------------------------------------------------------------
Project: Economic Insights DNSP Opex Cost Function
Author:  Michael Cunningham (based on a Denis Lawrence program), edited by Frontier Economics
File:    anBenchReg2.do
Initial Date:    4-Aug-2021
Revised Date:	 5-Apr-2022

Purpose: Real opex cost function using 4 specification (CD and TL, either using AUC or Capex to derive opex share) and 
	using 2 stochastic structures: SFA and LSE with panel-corrected SEs
--------------------------------------------------------------------------*/

* Preliminaries

foreach startyr in 2006 2012{
local endyr   2020

scalar define ppp_aust   		   = 1.527404363
scalar define ppp_nz    		   = 1.482877481
scalar define ppp_canada		   = 1.249076963
scalar define propex_rebase_aust   = 1.239047935
scalar define propex_rebase_nz     = 1.221665248
scalar define propex_rebase_canada = 1.160126855  

*****************************
* Capital cost

* Australia DNSPs
import excel "sources/DNSP AUC calculation (2020) FE.xlsx", sheet("FE_AUC") cellrange(B3:Q16) firstrow clear
foreach v of varlist C D E F G H I J K L M N O P Q {
   local x : variable label `v'
   rename `v' AUC`x'
}

reshape long AUC, i(DNSP)
replace AUC=AUC/1000
rename _j year
gen eiid_Char=substr(DNSP,1,2)
gen eiid=real(eiid_Char)
keep eiid year AUC
save "dta/AUS_AUC", replace

import excel "sources/DNSP AUC calculation (2020) FE.xlsx", sheet("FE_AUC") cellrange(B20:Q33) firstrow clear
foreach v of varlist C D E F G H I J K L M N O P Q {
   local x : variable label `v'
   rename `v' Capex`x'
}

reshape long Capex, i(DNSP)
replace Capex=Capex/1000
rename _j year
gen eiid_Char=substr(DNSP,1,2)
gen eiid=real(eiid_Char)
keep eiid year Capex
save "dta/AUS_Capex", replace

* Ontario DNSPs
* Ontario has a capital cost measure, which is a simplified return on capital plus depreciation measure. Is observed for 2013 onwards in individual spreadsheets, with a single sheet for earlier years.
use "dta/DNSPopex2", clear
keep if country==3
keep eiid country year opex
export delimited using "output/ForOntarioAUC.csv", replace

import excel "sources/2021-12-14 Ontario.xlsx", sheet("Calcs") cellrange("A1:K531") firstrow clear 
keep eiid_char year AUC Capex
rename AUC AUC_ONT
rename Capex Capex_ONT
replace Capex_ONT=. if Capex_ONT<0
save "dta/ONT_AUC", replace

* NZ DNSPs
use "dta/DNSPopex2", clear
keep if country==2
keep eiid country year opex
export delimited using "output/ForNZAUC.csv", replace

import excel "sources/2021-12-15 NewZealand.xlsx", sheet("ForNZAUC") cellrange("A1:L286") firstrow clear 
keep eiid year AUC Assetscommissioned
rename eiid eiid_char
rename AUC AUC_NZ
rename Assetscommissioned Capex_NZ
save "dta/NZ_AUC", replace

* Corrections due to CAM changes
import delimited "sources/2022-02-09 CurCamOpex.csv", clear
drop dnsp
save "dta/CurCamOpex", replace

* Read data (from 2021 Annual Benchmarking Report files)
use "dta/DNSPopex2", clear

* Merge with capital cost data
merge 1:1 eiid year using "dta/AUS_AUC"
tab _merge if country==1
drop _merge

merge 1:1 eiid year using "dta/AUS_Capex"
tab _merge if country==1
drop _merge

decode eiid, generate(eiid_char)
merge 1:1 eiid_char year using "dta/ONT_AUC"
tab _merge if country==3
replace AUC=AUC_ONT if AUC==.
replace Capex=Capex_ONT if Capex==.
drop _merge AUC_ONT Capex_ONT

merge 1:1 eiid_char year using "dta/NZ_AUC"
tab _merge if country==2
replace AUC=AUC_NZ if AUC==.
replace Capex=Capex_NZ if Capex==.
drop _merge eiid_char AUC_NZ Capex_NZ

gen y=-year
sort eiid y

drop y
sort eiid year

xtset dnsp year
qui tabulate dnsp, gen(d)
tabulate country, gen(jur)
correlate custnum circlen rmdem shareugc
tabulate year

* Sample selection
*-------------------------

drop if country == 3 & year < (`startyr' - 1)
drop if country < 3 & year < `startyr'
drop if country == 3 & year > (`endyr' - 1)
drop if country < 3 & year > `endyr'

* Modify & create variables
*--------------------------

generate yr=year
replace yr=yr+0.5 if dnsp==3
replace yr=yr+0.5 if dnsp==8
replace yr=yr+0.5 if dnsp==9
replace yr=yr+0.5 if dnsp==11
replace yr=yr+0.5 if dnsp==13
replace yr=yr-0.25 if country==2
replace yr=yr+0.5 if country==3

replace propex = propex / propex_rebase_aust    if country==1
replace propex = propex / propex_rebase_nz      if country==2
replace propex = propex / propex_rebase_canada  if country==3
replace propex = propex * ppp_nz / ppp_aust     if country==2
replace propex = propex * ppp_canada / ppp_aust if country==3

gen lastob = 0
replace lastob = 1 if dnsp != dnsp[_n+1]

* Regression log variables

gen lvc = log(opex / propex)
gen ly1 = log(custnum)
gen ly2 = log(circlen)
gen ly3 = log(rmdem)
gen lz1 = log(shareugc)

* mean correct output data
qui reg ly1
predict mly1
replace ly1=ly1-mly1
qui reg ly2
predict mly2
replace ly2=ly2-mly2
qui reg ly3
predict mly3
replace ly3=ly3-mly3

* TLG variables

gen ly11 = (ly1^2)/2
gen ly12 = ly1*ly2
gen ly13 = ly1*ly3
gen ly22 = (ly2^2)/2
gen ly23 = ly2*ly3
gen ly33 = (ly3^2)/2

merge 1:1 eiid year using "dta/CurCamOpex"
drop _merge
replace curcamopex=opex if curcamopex==.

gen Ratio_AUC=opex/(opex+AUC)
gen Ratio_Capex=opex/(curcamopex+Capex)

bysort dnsp: egen DNSP_Ratio_AUC=mean(Ratio_AUC)
bysort dnsp: egen DNSP_Ratio_Capex=mean(Ratio_Capex)

gen dnspcode="ACT" if eiid==1
replace dnspcode="AGD" if eiid==2
replace dnspcode="CIT" if eiid==3
replace dnspcode="END" if eiid==4
replace dnspcode="ENX" if eiid==5
replace dnspcode="ERG" if eiid==6
replace dnspcode="ESS" if eiid==7
replace dnspcode="JEN" if eiid==8
replace dnspcode="PCR" if eiid==9
replace dnspcode="SAP" if eiid==10
replace dnspcode="AND" if eiid==11
replace dnspcode="TND" if eiid==12
replace dnspcode="UED" if eiid==13

save "dta/iterationdata", replace

foreach myapproach in AUC Capex{
use "dta/iterationdata", clear

gen opexshare=log(DNSP_Ratio_`myapproach')

* Regression models
*---------------------- 

* 1. PCSECD

qui xtpcse lvc ly1 ly2 ly3 lz1 yr opexshare jur2 jur3 d2-d13 , c(a) het
estimates store PCSECD

putexcel set output/Results, sheet(LSECD`myapproach'`startyr') modify
putexcel A2=matrix(r(table)), names
putexcel A20=matrix(e(r2)), names

* Obtain efficiency scores from coeficients of dummy variables
gen eff_LSE_CD = 0 // i.e. ACT has a score of zero
forvalue j = 2/13{ // For other Aus DNSPs, score is taken from coefficient on relevant dummy
	replace eff_LSE_CD = _b[d`j'] if dnsp == `j'
}
qui sum eff_LSE_CD // Summarize so that minimum is able to be used next line
replace eff_LSE_CD = exp(-(eff_LSE_CD - r(min))) // Convert to efficiency scores, comparing to the min (including ACT's value of zero)

* 2. PCSETLG

qui xtpcse lvc ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr opexshare jur2 jur3 d2-d13, c(a) het
estimates store PCSETLG
	
putexcel set output/Results, sheet(LSETL`myapproach'`startyr') modify
putexcel A2=matrix(r(table)), names
putexcel A20=matrix(e(r2)), names
	
* Obtain efficiency scores from coeficients of dummy variables
gen eff_LSE_TL = 0 // i.e. ACT has a score of zero
forvalue j = 2/13{ // For other Aus DNSPs, score is taken from coefficient on relevant dummy
	replace eff_LSE_TL = _b[d`j'] if dnsp == `j'
}
qui sum eff_LSE_TL // Summarize so that minimum is able to be used next line
replace eff_LSE_TL = exp(-(eff_LSE_TL - r(min))) // Convert to efficiency scores, comparing to the min (including ACT's value of zero)

*test ly11 ly12 ly13 ly22 ly23 ly33
	
gen el1_pcse = _b[ly1] + _b[ly11]*ly1 + _b[ly12]*ly2 + _b[ly13]*ly3 
gen el2_pcse = _b[ly2] + _b[ly12]*ly1 + _b[ly22]*ly2 + _b[ly23]*ly3 
gen el3_pcse = _b[ly3] + _b[ly13]*ly1 + _b[ly23]*ly2 + _b[ly33]*ly3

display "ELASTICITIES"
tabstat el1_pcse el2_pcse el3_pcse, by(country) statistics(mean)
tabstat el1_pcse el2_pcse el3_pcse if dnsp <= 13, by(dnsp) statistics(mean)

display "MONOTONICITY VIOLATIONS - LSETLG `startyr' - `myapproach'"
gen mon1 = cond(el1_pcse < 0,1,0)	
gen mon2 = cond(el2_pcse < 0,1,0)	
gen mon3 = cond(el3_pcse < 0,1,0)
*tabstat mon1 mon2 mon3, by(country) statistics(mean)
tabstat mon1 mon2 mon3 if dnsp <= 13, by(dnsp) statistics(mean) 
drop mon1 mon2 mon3
	
* 3. SFACD

qui reg lvc ly1 ly2 ly3 lz1 yr opexshare jur2 jur3
matrix b0 = e(b),ln(e(rmse)^2), 0.1, 0.1
xtfrontier lvc ly1 ly2 ly3 lz1 yr opexshare jur2 jur3, cost from(b0, copy) ti
estimates store SFACD

putexcel set output/Results, sheet(SFACD`myapproach'`startyr') modify
putexcel A1=matrix(r(table)), names
putexcel A20=matrix(e(ll)), names

predict tecd, te
frontier_teci tecd
generate eff_SFA_CD = 1/tecd
*generate effcd_l = 1/tecd_u
*generate effcd_u = 1/tecd_l

*display "EFFICIENCY SCORES"
*list dnsp year effcd effcd_l effcd_u if lastob == 1 & dnsp <= 13
*tabstat effcd effcd_l effcd_u if dnsp <= 13, by(dnsp) statistics(mean)
*tabstat effcd if dnsp <= 13, by(dnsp) statistics(mean)

* 4. SFATLG
qui reg lvc ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr opexshare jur2 jur3
matrix b0 = e(b),ln(e(rmse)^2),0.1,0.1
if ("`myapproach'"=="AUC"){
if (`startyr'==2012){
matrix b0 = e(b),0.1,ln(e(rmse)^2),0.1 // Changed starting values due to convergence issue
}
}
xtfrontier lvc ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr opexshare jur2 jur3, ///
	cost from(b0, copy) ti
estimates store SFATLG
*test ly11 ly12 ly13 ly22 ly23 ly33
putexcel set output/Results, sheet(SFATL`myapproach'`startyr') modify
putexcel A1=matrix(r(table)), names
putexcel A20=matrix(e(ll)), names

predict tetl, te
frontier_teci tetl
generate eff_SFA_TL = 1/tetl
*tabstat eff_SFA_TL if dnsp<=13, by (dnsp) statistics(mean)
*generate efftl_l = 1/tetl_u
*generate efftl_u = 1/tetl_l

*display "EFFICIENCY SCORES"
*list dnsp year efftl efftl_l efftl_u if lastob == 1 & dnsp <= 13
*tabstat efftl efftl_l efftl_u if dnsp <= 13, by(dnsp) statistics(mean)
*tabstat efftl if dnsp <= 13, by(dnsp) statistics(mean)

gen el1_sfa = _b[ly1] + _b[ly11]*ly1 + _b[ly12]*ly2 + _b[ly13]*ly3 
gen el2_sfa = _b[ly2] + _b[ly12]*ly1 + _b[ly22]*ly2 + _b[ly23]*ly3 
gen el3_sfa = _b[ly3] + _b[ly13]*ly1 + _b[ly23]*ly2 + _b[ly33]*ly3

*display "ELASTICITIES"
*tabstat el1_sfa el2_sfa el3_sfa, by(country) statistics(mean)
*tabstat el1_sfa el2_sfa el3_sfa if dnsp <= 13, by(dnsp) statistics(mean)

display "MONOTONICITY VIOLATIONS - SFATLG `startyr' - `myapproach'"
gen mon1 = cond(el1_sfa < 0,1,0)	
gen mon2 = cond(el2_sfa < 0,1,0)	
gen mon3 = cond(el3_sfa < 0,1,0)
*tabstat mon1 mon2 mon3, by(country) statistics(mean)
tabstat mon1 mon2 mon3 if dnsp <= 13, by(dnsp) statistics(mean)
drop mon1 mon2 mon3

* LR test
*lrtest SFATLG SFACD

*display "EFFICIENCY SCORES"
tabstat eff_SFA_CD eff_SFA_TL eff_LSE_CD eff_LSE_TL if dnsp <= 13, by(dnsp) statistics(mean) format(%9.3fc)

rename eff_SFA_CD `myapproach'_eff_SFA_CD
rename eff_SFA_TL `myapproach'_eff_SFA_TL
rename eff_LSE_CD `myapproach'_eff_LSE_CD
rename eff_LSE_TL `myapproach'_eff_LSE_TL

keep eiid `myapproach'_eff_SFA_CD `myapproach'_eff_SFA_TL `myapproach'_eff_LSE_CD `myapproach'_eff_LSE_TL
duplicates drop
save "dta/`myapproach'_eff", replace

}

use "dta/AUC_eff", clear
merge 1:1 eiid using "dta/Capex_eff"
drop _merge
keep if eiid<=13
export excel using "output/Results.xlsx", sheet("Eff`startyr'",modify) first(var)
}
log close
exit
